"use strict";
var utils_1 = require("./utils");
function createGetterFn(pathKeys) {
    var fn = null;
    for (var i = pathKeys.length - 1; i >= 0; i--) {
        if (fn === null) {
            fn = finalFn(pathKeys[i]);
        }
        else {
            fn = stepFn(pathKeys[i], fn);
        }
    }
    return fn;
    function finalFn(key) {
        return function (scope, local) {
            if (local && local.hasOwnProperty(key))
                return local[key];
            if (scope)
                return scope[key];
        };
    }
    function stepFn(key, next) {
        return function (scope, local) {
            return next(scope && scope[key], local && local[key]);
        };
    }
}
function setterFn(scope, path, value) {
    var s = scope;
    var i = 0;
    for (; i < path.length - 1; i++) {
        if (utils_1.isUndefined(s[path[i]]) && i < path.length - 1) {
            s[path[i]] = {};
        }
        s = s[path[i]];
    }
    s[path[i]] = value;
    return scope;
}
/**
 * @description
 * return parse function
 * @returns {Function}
 */
function Parse() {
    var cache = {};
    return function (exp) {
        var fn = function () { };
        if (utils_1.isString(exp)) {
            var cacheKey = exp.trim();
            if (cacheKey in cache) {
                return cache[cacheKey];
            }
            var pathKeys = exp.split('.');
            fn = cache[cacheKey] = createGetterFn(pathKeys);
            fn.assign = function (scope, value) {
                return setterFn(scope, pathKeys, value);
            };
        }
        else if (utils_1.isFunction(exp)) {
            fn = function (scope, local) {
                return exp(scope, local);
            };
        }
        return fn;
    };
}
exports.Parse = Parse;
//# sourceMappingURL=parse.js.map