"use strict";
function isUndefined(value) {
    return typeof value === 'undefined';
}
exports.isUndefined = isUndefined;
function isNull(value) {
    return value === null;
}
exports.isNull = isNull;
function isNumber(value) {
    return typeof value === 'number';
}
exports.isNumber = isNumber;
function isString(value) {
    return typeof value === 'string';
}
exports.isString = isString;
function isObject(value) {
    return typeof value === 'object';
}
exports.isObject = isObject;
function isArray(value) {
    return Array.isArray(value);
}
exports.isArray = isArray;
function isFunction(value) {
    return typeof value === 'function';
}
exports.isFunction = isFunction;
function toArray(object) {
    return isArray(object) ? object : Object.keys(object).map(function (key) {
        return object[key];
    });
}
exports.toArray = toArray;
function equals(a, b) {
    if (a === b) {
        return true;
    }
    if (!(typeof a === 'object' && typeof b === 'object')) {
        return a === b;
    }
    var keysA = Object.keys(a);
    var keysB = Object.keys(b);
    if (keysA.length !== keysB.length) {
        return false;
    }
    // Test for A's keys different from B.
    var hasOwn = Object.prototype.hasOwnProperty;
    for (var i = 0; i < keysA.length; i++) {
        var key = keysA[i];
        if (!hasOwn.call(b, keysA[i]) || !equals(a[key], b[key])) {
            return false;
        }
    }
    return true;
}
exports.equals = equals;
function objectContains(partial, object) {
    return Object.keys(partial).every(function (key) {
        return key in object && object[key] == partial[key];
    });
}
exports.objectContains = objectContains;
function deepKeys(obj, stack, parent, intermediate) {
    if (stack === void 0) { stack = []; }
    if (parent === void 0) { parent = null; }
    if (intermediate === void 0) { intermediate = false; }
    Object.keys(obj).forEach(function (el) {
        // Escape . in the element name
        var escaped = el.replace(/\./g, '\\\.');
        // If it's a nested object
        if (isObject(obj[el]) && !isArray(obj[el])) {
            // Concatenate the new parent if exist
            var p = parent ? parent + '.' + el : parent;
            // Push intermediate parent key if flag is true
            if (intermediate)
                stack.push(parent ? p : escaped);
            deepKeys(obj[el], stack, p || escaped, intermediate);
        }
        else {
            // Create and save the key
            var key = parent ? parent + '.' + escaped : escaped;
            stack.push(key);
        }
    });
    return stack;
}
exports.deepKeys = deepKeys;
/**
 * @description
 * return the first n element of an array,
 * if expression provided, is returns as long the expression return truthy
 * @param array
 * @param n {number}
 * @param expression {$parse}
 * @return array or single object
 */
function getFirstMatches(array, n, getter) {
    var count = 0;
    return array.filter(function (elm) {
        var rest = !isUndefined(getter) ? count < n && getter(elm) : count < n;
        count = rest ? count + 1 : count;
        return rest;
    });
}
exports.getFirstMatches = getFirstMatches;
/**
 * @description
 * search for approximate pattern in string
 * @param word
 * @param pattern
 * @returns {*}
 */
function hasApproxPattern(word, pattern) {
    function indexOf(word, p, c) {
        var j = 0;
        while ((p + j) <= word.length) {
            if (word.charAt(p + j) == c)
                return j;
            j++;
        }
        return -1;
    }
    var p = 0;
    for (var i = 0; i <= pattern.length; i++) {
        var index = indexOf(word, p, pattern.charAt(i));
        if (index == -1)
            return false;
        p += index + 1;
    }
    return true;
}
exports.hasApproxPattern = hasApproxPattern;
function convertToDecimal(num, decimal) {
    return Math.round(num * Math.pow(10, decimal)) / (Math.pow(10, decimal));
}
exports.convertToDecimal = convertToDecimal;
//# sourceMappingURL=utils.js.map